//
//  AAILoadingView.h
//  AAILivenessUI
//
//  Created by loong on 2024/7/10.
//

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface AAILoadView : UIView

@property(nonatomic) UIColor *brandColor;

- (void)startAnimation;

@end

NS_ASSUME_NONNULL_END
