//
//  AAILoadingHud.h
//  AAILivenessUI
//
//  Created by loong on 2024/7/10.
//

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface AAILoadingHud : UIView

- (instancetype)initWithText:(NSString *)text
                  brandColor:(UIColor *)brandColor;

- (instancetype)initWithText:(NSString *)text
                  brandColor:(UIColor *)brandColor
                        font:(UIFont * _Nullable)font;

- (void)showInView:(UIView *)view;
+ (void)dismissInView:(UIView *)view;

- (void)updateTitleLabelAccId:(NSString *)accessibilityIdentifier;

@end

NS_ASSUME_NONNULL_END
