//
//  UIColor+AAILD.h
//  AAILivenessUI
//
//  Created by loong on 2024/7/15.
//

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface UIColor (AAILD)

+ (UIColor *)aaild_colorWithHex:(NSInteger)hex;

+ (UIColor *)aaild_brandColor;

@end

NS_ASSUME_NONNULL_END
