#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface UIView (AAILD)

- (UIView *)aaild_safeMarginToView:(UIView *)toView
                            t:(NSNumber * _Nullable)top
                            l:(NSNumber * _Nullable)left
                            b:(NSNumber * _Nullable)bottom
                            r:(NSNumber * _Nullable)right;

- (UIView *)aaild_marginToView:(UIView *)toView
                            t:(NSNumber * _Nullable)top
                            l:(NSNumber * _Nullable)left
                            b:(NSNumber * _Nullable)bottom
                            r:(NSNumber * _Nullable)right;

- (UIView *)aaild_width:(NSNumber * _Nullable)width height:(NSNumber * _Nullable)height;

@end

NS_ASSUME_NONNULL_END
