//
//  UIColor+AAILD.m
//  AAILivenessUI
//
//  Created by loong on 2024/7/15.
//

#import "UIColor+AAILD.h"

@implementation UIColor (AAILD)

+ (UIColor *)aaild_colorWithHex:(NSInteger)hex {
    return [UIColor colorWithRed:((hex & 0xFF0000) >> 16) / 255.0
                           green:((hex & 0xFF00) >> 8) / 255.0
                            blue:(hex & 0xFF) / 255.0
                           alpha:1.0];
}

+ (UIColor *)aaild_brandColor {
    return [UIColor aaild_colorWithHex: 0x5BC413];
}

@end
