//
//  AppDelegate.swift
//  LivenessSDKSwiftDemo
//
//  Created by advance on 2021/7/21.
//

import UIKit
//import ShowHow
@main
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        window = UIWindow(frame: UIScreen.main.bounds)
        window?.backgroundColor = .white
        window?.makeKeyAndVisible()
        let navc = UINavigationController(rootViewController: ViewController())
        navc.isNavigationBarHidden = true
        window?.rootViewController = navc
        
        return true
    }

    @available(iOS 13.0, *)
    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        // Called when a new scene session is being created.
        // Use this method to select a configuration to create the new scene with.
        let confi = UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
        confi.delegateClass = SceneDelegate.self
        confi.storyboard = UIStoryboard(name: "Main", bundle: nil)
        
        return confi
    }
}

